@@ -8,7 +8,7 @@ build_time_vars = {'ABIFLAGS': '',
  'ALIGNOF_SIZE_T': 4,
  'ALT_SOABI': 0,
  'ANDROID_API_LEVEL': 0,
- 'AR': 'hppa2.0-unknown-linux-gnu-ar',
+ 'AR': 'hppa1.1-unknown-linux-gnu-ar',
  'ARFLAGS': 'rcs',
  'BASECFLAGS': '-fno-strict-overflow -Wsign-compare',
  'BASECPPFLAGS': '',
@@ -16,14 +16,14 @@ build_time_vars = {'ABIFLAGS': '',
  'BINDIR': '/usr/bin',
  'BINLIBDEST': '/usr/lib/python3.12',
  'BLDLIBRARY': '-L. -lpython3.12',
- 'BLDSHARED': 'hppa2.0-unknown-linux-gnu-gcc -shared',
+ 'BLDSHARED': 'hppa1.1-unknown-linux-gnu-gcc -shared',
  'BOOTSTRAP_HEADERS': '\\',
  'BUILDEXE': '',
  'BUILDPYTHON': 'python',
- 'BUILD_GNU_TYPE': 'hppa2.0-unknown-linux-gnu',
+ 'BUILD_GNU_TYPE': 'hppa1.1-unknown-linux-gnu',
  'BUILD_SCRIPTS_DIR': 'build/scripts-3.12',
  'BYTESTR_DEPS': '\\',
- 'CC': 'hppa2.0-unknown-linux-gnu-gcc',
+ 'CC': 'hppa1.1-unknown-linux-gnu-gcc',
  'CCSHARED': '-fPIC',
  'CFLAGS': '-fno-strict-overflow -Wsign-compare -DNDEBUG',
  'CFLAGSFORSHARED': '-fPIC',
@@ -35,7 +35,7 @@ build_time_vars = {'ABIFLAGS': '',
  'CONFIGFILES': 'configure configure.ac acconfig.h pyconfig.h.in '
                 'Makefile.pre.in',
  'CONFIGURE_CFLAGS': '',
- 'CONFIGURE_CFLAGS_NODIST': '-O2 -pipe -march=2.0 -fwrapv -std=c11 -Wextra '
+ 'CONFIGURE_CFLAGS_NODIST': '-O2 -pipe -march=1.1 -fwrapv -std=c11 -Wextra '
                             '-Wno-unused-parameter '
                             '-Wno-missing-field-initializers '
                             '-Wstrict-prototypes '
@@ -45,39 +45,39 @@ build_time_vars = {'ABIFLAGS': '',
  'CONFIGURE_LDFLAGS': '',
  'CONFIGURE_LDFLAGS_NODIST': '-Wl,-O1 -Wl,--as-needed',
  'CONFIGURE_LDFLAGS_NOLTO': '',
- 'CONFIG_ARGS': "'--prefix=/usr' '--build=hppa2.0-unknown-linux-gnu' "
-                "'--host=hppa2.0-unknown-linux-gnu' '--mandir=/usr/share/man' "
+ 'CONFIG_ARGS': "'--prefix=/usr' '--build=hppa1.1-unknown-linux-gnu' "
+                "'--host=hppa1.1-unknown-linux-gnu' '--mandir=/usr/share/man' "
                 "'--infodir=/usr/share/info' '--datadir=/usr/share' "
                 "'--sysconfdir=/etc' '--localstatedir=/var/lib' "
                 "'--datarootdir=/usr/share' "
-                "'--docdir=/usr/share/doc/python-3.12.1' "
-                "'--htmldir=/usr/share/doc/python-3.12.1/html' "
+                "'--docdir=/usr/share/doc/python-3.12.2_p1' "
+                "'--htmldir=/usr/share/doc/python-3.12.2_p1/html' "
                 "'--libdir=/usr/lib' 'ac_cv_header_stropts_h=no' "
                 "'--enable-shared' '--without-static-libpython' "
                 "'--enable-ipv6' '--infodir=/usr/share/info' "
                 "'--mandir=/usr/share/man' '--with-computed-gotos' "
                 "'--with-dbmliborder=gdbm' '--with-libc=' "
                 "'--enable-loadable-sqlite-extensions' '--without-ensurepip' "
-                "'--with-system-expat' '--with-platlibdir=lib' "
+                "'--without-lto' '--with-system-expat' '--with-platlibdir=lib' "
                 "'--with-pkg-config=yes' "
                 "'--with-wheel-pkg-dir=/usr/lib/python/ensurepip' "
-                "'--without-assertions' '--without-lto' "
-                "'--disable-optimizations' '--with-readline=readline' "
-                "'--without-valgrind' 'build_alias=hppa2.0-unknown-linux-gnu' "
-                "'host_alias=hppa2.0-unknown-linux-gnu' "
-                "'PKG_CONFIG=hppa2.0-unknown-linux-gnu-pkg-config' 'CFLAGS=' "
+                "'--without-assertions' '--disable-optimizations' "
+                "'--with-readline=readline' '--without-valgrind' "
+                "'build_alias=hppa1.1-unknown-linux-gnu' "
+                "'host_alias=hppa1.1-unknown-linux-gnu' "
+                "'PKG_CONFIG=hppa1.1-unknown-linux-gnu-pkg-config' 'CFLAGS=' "
                 "'LDFLAGS=' 'CPPFLAGS= -I/usr/include/ncursesw'",
  'CONFINCLUDEDIR': '/usr/include',
  'CONFINCLUDEPY': '/usr/include/python3.12',
  'COREPYTHONPATH': '',
- 'COVERAGE_INFO': '/var/tmp/portage/dev-lang/python-3.12.1/work/Python-3.12.1/coverage.info',
+ 'COVERAGE_INFO': '/var/tmp/portage/dev-lang/python-3.12.2_p1/work/Python-3.12.2/coverage.info',
  'COVERAGE_LCOV_OPTIONS': '--rc lcov_branch_coverage=1',
- 'COVERAGE_REPORT': '/var/tmp/portage/dev-lang/python-3.12.1/work/Python-3.12.1/lcov-report',
+ 'COVERAGE_REPORT': '/var/tmp/portage/dev-lang/python-3.12.2_p1/work/Python-3.12.2/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--rc lcov_branch_coverage=1 --branch-coverage '
                             '--title "CPython 3.12 LCOV report [commit $(shell '
                             ')]"',
  'CPPFLAGS': '-I. -I./Include -I/usr/include/ncursesw',
- 'CXX': 'hppa2.0-unknown-linux-gnu-g++',
+ 'CXX': 'hppa1.1-unknown-linux-gnu-g++',
  'DEEPFREEZE_C': 'Python/deepfreeze/deepfreeze.c',
  'DEEPFREEZE_DEPS': './Tools/build/deepfreeze.py _bootstrap_python '
                     './Programs/_freeze_module.py \\',
@@ -145,7 +145,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_BLUETOOTH_BLUETOOTH_H': 0,
  'HAVE_BLUETOOTH_H': 0,
  'HAVE_BROKEN_MBSTOWCS': 0,
- 'HAVE_BROKEN_NICE': 1,
+ 'HAVE_BROKEN_NICE': 0,
  'HAVE_BROKEN_PIPE_BUF': 0,
  'HAVE_BROKEN_POLL': 0,
  'HAVE_BROKEN_POSIX_SEMAPHORES': 0,
@@ -511,7 +511,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_STRFTIME': 1,
  'HAVE_STRINGS_H': 1,
  'HAVE_STRING_H': 1,
- 'HAVE_STRLCPY': 0,
+ 'HAVE_STRLCPY': 1,
  'HAVE_STROPTS_H': 0,
  'HAVE_STRSIGNAL': 1,
  'HAVE_STRUCT_PASSWD_PW_GECOS': 1,
@@ -618,7 +618,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_ZLIB_H': 0,
  'HAVE__GETPTY': 0,
  'HOSTRUNNER': '',
- 'HOST_GNU_TYPE': 'hppa2.0-unknown-linux-gnu',
+ 'HOST_GNU_TYPE': 'hppa1.1-unknown-linux-gnu',
  'INCLDIRSTOMAKE': '/usr/include /usr/include /usr/include/python3.12 '
                    '/usr/include/python3.12',
  'INCLUDEDIR': '/usr/include',
@@ -631,19 +631,19 @@ build_time_vars = {'ABIFLAGS': '',
  'INSTSONAME': 'libpython3.12.so.1.0',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
- 'LDCXXSHARED': 'hppa2.0-unknown-linux-gnu-g++ -shared',
+ 'LDCXXSHARED': 'hppa1.1-unknown-linux-gnu-g++ -shared',
  'LDFLAGS': '',
  'LDFLAGS_NODIST': '',
  'LDLIBRARY': 'libpython3.12.so',
  'LDLIBRARYDIR': '',
- 'LDSHARED': 'hppa2.0-unknown-linux-gnu-gcc -shared',
+ 'LDSHARED': 'hppa1.1-unknown-linux-gnu-gcc -shared',
  'LDVERSION': '3.12',
  'LIBC': '',
  'LIBDEST': '/usr/lib/python3.12',
  'LIBDIR': '/usr/lib',
  'LIBEXPAT_A': 'Modules/expat/libexpat.a',
  'LIBEXPAT_CFLAGS': '-fno-strict-overflow -Wsign-compare -DNDEBUG -O2 -pipe '
-                    '-march=2.0 -fwrapv -std=c11 -Wextra -Wno-unused-parameter '
+                    '-march=1.1 -fwrapv -std=c11 -Wextra -Wno-unused-parameter '
                     '-Wno-missing-field-initializers -Wstrict-prototypes '
                     '-Werror=implicit-function-declaration '
                     '-fvisibility=hidden  -I./Include/internal -I. -I./Include '
@@ -652,7 +652,7 @@ build_time_vars = {'ABIFLAGS': '',
  'LIBEXPAT_OBJS': '\\',
  'LIBHACL_CFLAGS': '-I./Modules/_hacl/include -D_BSD_SOURCE -D_DEFAULT_SOURCE '
                    '-fno-strict-overflow -Wsign-compare -DNDEBUG -O2 -pipe '
-                   '-march=2.0 -fwrapv -std=c11 -Wextra -Wno-unused-parameter '
+                   '-march=1.1 -fwrapv -std=c11 -Wextra -Wno-unused-parameter '
                    '-Wno-missing-field-initializers -Wstrict-prototypes '
                    '-Werror=implicit-function-declaration -fvisibility=hidden  '
                    '-I./Include/internal -I. -I./Include '
@@ -665,7 +665,7 @@ build_time_vars = {'ABIFLAGS': '',
  'LIBMPDEC_A': 'Modules/_decimal/libmpdec/libmpdec.a',
  'LIBMPDEC_CFLAGS': '-I./Modules/_decimal/libmpdec -DCONFIG_32=1 -DANSI=1 '
                     '-fno-strict-overflow -Wsign-compare -DNDEBUG -O2 -pipe '
-                    '-march=2.0 -fwrapv -std=c11 -Wextra -Wno-unused-parameter '
+                    '-march=1.1 -fwrapv -std=c11 -Wextra -Wno-unused-parameter '
                     '-Wno-missing-field-initializers -Wstrict-prototypes '
                     '-Werror=implicit-function-declaration '
                     '-fvisibility=hidden  -I./Include/internal -I. -I./Include '
@@ -683,7 +683,7 @@ build_time_vars = {'ABIFLAGS': '',
  'LIBRARY_OBJS_OMIT_FROZEN': '\\',
  'LIBS': '-ldl',
  'LIBSUBDIRS': 'asyncio \\',
- 'LINKCC': 'hppa2.0-unknown-linux-gnu-gcc',
+ 'LINKCC': 'hppa1.1-unknown-linux-gnu-gcc',
  'LINKFORSHARED': '-Xlinker -export-dynamic',
  'LINK_PYTHON_DEPS': 'libpython3.12.so libpython3.so',
  'LINK_PYTHON_OBJS': '-L. -lpython3.12',
@@ -702,7 +702,7 @@ build_time_vars = {'ABIFLAGS': '',
  'MAJOR_IN_SYSMACROS': 1,
  'MAKESETUP': './Modules/makesetup',
  'MANDIR': '/usr/share/man',
- 'MKDIR_P': '/bin/mkdir -p',
+ 'MKDIR_P': '/usr/bin/mkdir -p',
  'MODBUILT_NAMES': 'array  _asyncio  _bisect  _contextvars  _csv  _heapq  '
                    '_json  _lsprof  _opcode  _pickle  _queue  _random  '
                    '_struct  _xxsubinterpreters  _xxinterpchannels  _zoneinfo  '
@@ -867,7 +867,7 @@ build_time_vars = {'ABIFLAGS': '',
  'MODULE__CSV_STATE': 'yes',
  'MODULE__CTYPES_CFLAGS': '-fno-strict-overflow -I/usr/lib/libffi/include',
  'MODULE__CTYPES_DEPS': './Modules/_ctypes/ctypes.h',
- 'MODULE__CTYPES_LDFLAGS': '-lffi  -ldl',
+ 'MODULE__CTYPES_LDFLAGS': '-lffi -ldl',
  'MODULE__CTYPES_MALLOC_CLOSURE': '',
  'MODULE__CTYPES_STATE': 'yes',
  'MODULE__CTYPES_TEST_LDFLAGS': '-lm',
@@ -876,7 +876,7 @@ build_time_vars = {'ABIFLAGS': '',
  'MODULE__CURSES_LDFLAGS': '-lncursesw -ltinfow',
  'MODULE__CURSES_PANEL_CFLAGS': '-I/usr/include/ncursesw -D_DEFAULT_SOURCE  '
                                 '-D_DEFAULT_SOURCE  -I/usr/include/ncursesw',
- 'MODULE__CURSES_PANEL_LDFLAGS': '-lpanelw  -lncursesw -ltinfow',
+ 'MODULE__CURSES_PANEL_LDFLAGS': '-lpanelw -lncursesw -ltinfow',
  'MODULE__CURSES_PANEL_STATE': 'yes',
  'MODULE__CURSES_STATE': 'yes',
  'MODULE__DATETIME_LDFLAGS': '-lm',
@@ -1033,7 +1033,7 @@ build_time_vars = {'ABIFLAGS': '',
  'PYTHON_OBJS': '\\',
  'PY_BUILTIN_HASHLIB_HASHES': '"md5,sha1,sha2,sha3,blake2"',
  'PY_BUILTIN_MODULE_CFLAGS': '-fno-strict-overflow -Wsign-compare -DNDEBUG -O2 '
-                             '-pipe -march=2.0 -fwrapv -std=c11 -Wextra '
+                             '-pipe -march=1.1 -fwrapv -std=c11 -Wextra '
                              '-Wno-unused-parameter '
                              '-Wno-missing-field-initializers '
                              '-Wstrict-prototypes '
@@ -1042,14 +1042,14 @@ build_time_vars = {'ABIFLAGS': '',
                              '-I./Include -I/usr/include/ncursesw -fPIC '
                              '-DPy_BUILD_CORE_BUILTIN',
  'PY_CFLAGS': '-fno-strict-overflow -Wsign-compare -DNDEBUG',
- 'PY_CFLAGS_NODIST': '-O2 -pipe -march=2.0 -fwrapv -std=c11 -Wextra '
+ 'PY_CFLAGS_NODIST': '-O2 -pipe -march=1.1 -fwrapv -std=c11 -Wextra '
                      '-Wno-unused-parameter -Wno-missing-field-initializers '
                      '-Wstrict-prototypes '
                      '-Werror=implicit-function-declaration '
                      '-fvisibility=hidden  -I./Include/internal',
  'PY_COERCE_C_LOCALE': 1,
  'PY_CORE_CFLAGS': '-fno-strict-overflow -Wsign-compare -DNDEBUG -O2 -pipe '
-                   '-march=2.0 -fwrapv -std=c11 -Wextra -Wno-unused-parameter '
+                   '-march=1.1 -fwrapv -std=c11 -Wextra -Wno-unused-parameter '
                    '-Wno-missing-field-initializers -Wstrict-prototypes '
                    '-Werror=implicit-function-declaration -fvisibility=hidden  '
                    '-I./Include/internal -I. -I./Include '
@@ -1066,7 +1066,7 @@ build_time_vars = {'ABIFLAGS': '',
  'PY_SSL_DEFAULT_CIPHERS': 1,
  'PY_SSL_DEFAULT_CIPHER_STRING': 0,
  'PY_STDMODULE_CFLAGS': '-fno-strict-overflow -Wsign-compare -DNDEBUG -O2 '
-                        '-pipe -march=2.0 -fwrapv -std=c11 -Wextra '
+                        '-pipe -march=1.1 -fwrapv -std=c11 -Wextra '
                         '-Wno-unused-parameter -Wno-missing-field-initializers '
                         '-Wstrict-prototypes '
                         '-Werror=implicit-function-declaration '
@@ -1083,7 +1083,7 @@ build_time_vars = {'ABIFLAGS': '',
  'READELF': '@READELF@',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/var/tmp/portage/dev-lang/python-3.12.1/work/Python-3.12.1',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/var/tmp/portage/dev-lang/python-3.12.2_p1/work/Python-3.12.2',
  'SCRIPTDIR': '/usr/lib',
  'SCRIPT_2TO3': 'build/scripts-3.12/2to3-3.12',
  'SCRIPT_IDLE': 'build/scripts-3.12/idle3.12',
@@ -1207,10 +1207,10 @@ build_time_vars = {'ABIFLAGS': '',
  'SYS_SELECT_WITH_SYS_TIME': 1,
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/var/tmp/portage/dev-lang/python-3.12.1/work/Python-3.12.1 '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/var/tmp/portage/dev-lang/python-3.12.2_p1/work/Python-3.12.2 '
                './python -E',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/var/tmp/portage/dev-lang/python-3.12.1/work/Python-3.12.1 '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/var/tmp/portage/dev-lang/python-3.12.2_p1/work/Python-3.12.2 '
                './python -E ./Tools/scripts/run_tests.py',
  'TESTSUBDIRS': 'idlelib/idle_test \\',
  'TESTTIMEOUT': 1200,
@@ -1240,8 +1240,8 @@ build_time_vars = {'ABIFLAGS': '',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/var/tmp/portage/dev-lang/python-3.12.1/work/Python-3.12.1',
- 'abs_srcdir': '/var/tmp/portage/dev-lang/python-3.12.1/work/Python-3.12.1',
+ 'abs_builddir': '/var/tmp/portage/dev-lang/python-3.12.2_p1/work/Python-3.12.2',
+ 'abs_srcdir': '/var/tmp/portage/dev-lang/python-3.12.2_p1/work/Python-3.12.2',
  'datarootdir': '/usr/share',
  'exec_prefix': '/usr',
  'prefix': '/usr',
